/*
 * GraphLCD driver library
 *
 * dbox.h  -  stream2dbox
 *
 * This file is released under the GNU General Public License. Refer
 * to the COPYING file distributed with this package.
 *
 * (c) 2004 Stephan Skrodzki 
 */

#include <fcntl.h>
#include <stdio.h>
#include <syslog.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

#include "common.h"
#include "config.h"
#include "dbox.h"

#define TCP_PORT 20003

namespace GLCD
{

cDriverDbox::cDriverDbox(cDriverConfig * config)
:	config(config)
{
	oldConfig = new cDriverConfig(*config);
}

cDriverDbox::~cDriverDbox()
{
	delete oldConfig;
}

int cDriverDbox::Init()
{
	height = config->height ? config->height : 64;
	width = config->width ? config->width : 120;
	port = config->port ? config->port : 20003;
	
	screensize = width * height / 8;
	offbuff = new char[screensize];
	if (!offbuff)
	{
		syslog(LOG_ERR, "%s: failed to alloc memory for dbox display.\n", config->name.c_str());
		return -1;
	}

	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	serv_addr.sin_port = htons(port);
	
	sock_server = socket(AF_INET, SOCK_STREAM, 0);
	
	if (bind(sock_server, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) == -1)
	{
		syslog(LOG_ERR, "%s: bind to socket failed!\n", config->name.c_str());
		sock_server = 0;
	}
	else
	{
		if (listen(sock_server, 3) == -1)
		{
			sock_server = 0;
			syslog(LOG_ERR, "%s: listen failed!\n", config->name.c_str());
		}
	  	else
	  	{
	
			fcntl(sock_server, F_SETFL, O_NONBLOCK);
		}
	}	
	sock_dbox = 0;

        

	*oldConfig = *config;
	
	// clear display
	Clear();
	
	Refresh(true);
	
	if (sock_server)
		syslog(LOG_INFO, "%s: Dbox display initialized: Port %d, width %d, height %d .\n", config->name.c_str(), port, width, height);
	
	return 0;
}

int cDriverDbox::DeInit()
{
	if (sock_dbox > 0)
	{
		close(sock_dbox);
		syslog(LOG_INFO, "%s: closing sock_dbox!\n", config->name.c_str());
		sock_dbox = 0;
	}

	if (sock_server > 0)
	{
		close(sock_server);
		syslog(LOG_INFO, "%s: closing sock_server!\n", config->name.c_str());
		sock_server = 0;
	}

	if (offbuff);
		delete[] offbuff;
	
	return 0;
}

int cDriverDbox::CheckSetup()
{
	if (config->port != oldConfig->port ||
	    config->width != oldConfig->width ||
	    config->height != oldConfig->height)
	{
		DeInit();
		Init();
		return 0;
	}

	if (config->upsideDown != oldConfig->upsideDown ||
	    config->invert != oldConfig->invert)
	{
		oldConfig->upsideDown = config->upsideDown;
		oldConfig->invert = config->invert;
		return 1;
	}
	return 0;
}

void cDriverDbox::SetPixel(int x, int y)
{
	if (x >= width || y >= height)
		return;

	if (config->upsideDown)
	{
		x = width - 1 - x;
		y = height - 1 - y;
	}

	// Figure out where in memory to put the pixel
	int location = x + y * width;
	
	*(offbuff + location/8) |= 1 << location % 8;

}

void cDriverDbox::Clear()
{
	memset(offbuff, 0, screensize);
}

void cDriverDbox::Set8Pixels(int x, int y, unsigned char data)
{
	int n;
	x &= 0xFFF8;

	for (n = 0; n < 8; ++n)
	{
		
		if ( (data & (0x80 >> n)) && !config->invert)
			SetPixel(x + n, y);
		else if ( !(data & (0x80 >> n)) && config->invert) 
			SetPixel(x + n, y);

	}
}

void cDriverDbox::Refresh(bool refreshAll)
{
	struct sockaddr_in remote_host;
	socklen_t sin_size = sizeof (sockaddr_in);
	
	if (sock_server > 0)
	{
		printf("LCD: Listening %d %d!\n", sock_server, sock_dbox);
		
		if (sock_dbox <= 0)
			sock_dbox = accept(sock_server, (sockaddr *) &remote_host, &sin_size);
				
		if (sock_dbox > 0)
		{
			int len = send(sock_dbox, offbuff, screensize, 0);
//			syslog(LOG_INFO, "%s: Sent %d Bytes\n", config->name.c_str(), len);
			
			if (len < 0)
			{
				syslog(LOG_INFO, "%s: Connection closed!\n", config->name.c_str());
				close(sock_dbox);
				sock_dbox = 0;
			}
		}
	}
	
}

} // end of namespace
